#!/bin/bash
# Licensed Materials - Property of IBM
#
# 09N6996
#
# (C) Copyright IBM Corp. 2012 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.

PATH=/bin:/usr/bin:/usr/sbin/rsct/bin:/opt/ibm/director/ha/bin
PROGNAME=$(basename $0)
RUN_AS_ROOT=/opt/ibm/director/vmi/bin/execWithPrivilege
COMMON_CODE=/opt/ibm/director/ha/bin/include/smhacommon.sh
INSTALL_DIR=/opt/ibm/director
CMD_DIR=$INSTALL_DIR/ha/bin/priv          # required by smhacommon.sh

if [ -f $COMMON_CODE ]; then
    source $COMMON_CODE
else
    echo "$PROGNAME:Common code not found."
    exit 1
fi
#############################################################

# this is a plugin script invoked by the chconfig command. Its purpose is to check
# if the --datetime argument was specified to chconfig and, if so, is the time being
# set backward. If the time is set backward, the TSA and/or RSCT stack must by
# recycled. The stack is stopped in the "pre" phase and restarted in the "post"
# or "cleanup" phase

# If Director HA is configured, TSA automation is turned off to prevent the TSA
# resources from being restarted.

SMHACHKTIME_ACQUIRED_LOCK=no

log_and_exit () {

    if [ "$SMHACHKTIME_ACQUIRED_LOCK" = "yes" ]; then
        rw_lock write free CmdSMHA $PROGNAME
    fi

    # call sm_msg to put the message to the system log and HA log, but redirect STDOUT to NULL

    sm_msg "$*" > /dev/null
    exit 1
}

printerrmsg () {
    local msg

    case $1 in
        1 ) msg="`eval_gettext "No value is specified for the --datetime parameter. Specify a value of the form MMDDhhmm[[CC]YY][.ss] and try the command again."`" ;;
        2 ) msg="`eval_gettext "The value specified for the --datetime parameter is invalid. Specify a value of the form MMDDhhmm[[CC]YY][.ss]. Retry the command with a valid value."`" ;;
        3 ) msg="`eval_gettext "The system cannot disable the high availability environment. Retry the operation. If the problem persists, reboot the node and then try the operation again. If you continue to see this message, collect logs by running pedbg -c -q4 and contact Software Support."`" ;;
        4 ) msg="`eval_gettext "Invalid arguments were passed to a plug-in script. Collect logs by running pedbg -c -q4, then contact Software Support."`" ;;
        5 ) msg="`eval_gettext "The system time cannot be changed now. Retry this command in a few minutes."`" ;;
    esac

    echo "$msg" 1>&2     # send it to STDERR
}

# the arguments passed to the following function are the arguments passed to the
# chconfig command. Ignore all but ---datetime
# The function returns 0 if time is being reversed, i.e. the function "succeeds"

chk_for_negative_time () {
    local Count=
    local CurrentTime
    local Time=
    local Seconds=
    local MM=
    local DD=
    local hh=
    local mm=
    local Year=
    local year=
    local invalidTimeSpec=0
    local DateSpec="%m%d%"

    DateSpec="${DateSpec}H%M"    # we must construct the DateSpec string in parts since '%' 'H' '%' is a SCCS keyword and will get expanded
    while [ $# -gt 0 ] ; do

        if [ $1 = "--datetime" ]; then
            if [ -z "$2" ]; then
                printerrmsg 1
                log_and_exit "No value specified for --datetime."
            fi

            invalidTimeSpec=1            # assume the worst

            # the format of the value for --datetime is MMDDhhmm[[CC]YY][.ss]

            # check if seconds are specified in the parameter value

            Count=$(expr $2 : '[0-9]\+\.[0-9][0-9]$')
            if [ -n "$Count" -a $Count -gt 0 ]; then
                Time=${2%.*}
                Seconds=${2#*.}

                if [ $Seconds -gt 60 ]; then
                    break
                fi

                DateSpec=${DateSpec}%S
            else
                Time=$2
            fi

            # either seconds were not specified, or have been stripped off; now check that the Time is only digits

            Count=$(expr $Time : '[0-9]\+$')
            if [ -n "$Count" -a $Count -gt 0 ]; then

                # Time should be one of MMDDhhmm or MMDDhhmmYY or MMDDhhmmCCYY; check by counting the number of chars

                Count=${#Time}
                if [ $Count -eq 8 -o $Count -eq 10 -o $Count -eq 12 ]; then

                    MM=${Time:0:2}
                    if [ $MM -lt 1 -o $MM -gt 12 ]; then     # validate Month of year
                        break
                    fi

                    DD=${Time:2:2}
                    if [ $DD -lt 1 -o $DD -gt 31 ]; then     # validate Day of month
                        break
                    fi

                    hh=${Time:4:2}
                    if [ $hh -gt 23 ]; then                  # validate Hour of day
                        break
                    fi

                    mm=${Time:6:2}
                    if [ $mm -gt 59 ]; then                  # validate Minute of hour
                        break
                    fi

                    # At this point validation is complete, as much as can reasonably be done

                    debug "--datetime parameter value: Time=$Time  Seconds=$Seconds"

                    if [ $Count -eq 10 ]; then
                        year=${Time:8:2}            # get "short" year

                        # make into 4 digit year, per GNU date syntax rules
                        if [[ $year < 68 ]]; then
                            Year=20${year}
                        else
                            Year=19${year}
                        fi
                        DateSpec=%Y${DateSpec}

                    elif [ $Count -eq 12 ]; then
                        Year=${Time:8:4}            # get "long" year
                        DateSpec=%Y${DateSpec}
                    fi

                    Time=${Year}${MM}${DD}${hh}${mm}${Seconds}
                    CurrentTime=$(date +${DateSpec})

                    debug "Time=$Time   DateSpec=$DateSpec   CurrentTime=$CurrentTime"

                    if [[ $Time < $CurrentTime ]]; then
                        debug "Time being set back"
                        return 0
                    elif [[ $Time = $CurrentTime ]]; then
                        # if the time is specified with minute accuracy, rather than to the second,
                        # then we can't tell if time is going to move forward or backward. So when
                        # equal assume time is moving backward.

                        debug "New time and current time are equal"
                        return 0
                    else
                        debug "Time being set forward"
                        return 1
                    fi
                fi
            fi

            break     # we are done
        fi

        shift
    done

    if [ $invalidTimeSpec -eq 1 ]; then
        printerrmsg 2
        log_and_exit "The value specified for --datetime is invalid."
    fi
    return 1
}

# the following function disables TSA automation, checking that it could successfully be disabled
# This function also disables critical resource protection

stop_automation () {
    local rc
    local result

    result=$(call_tsa_samctrl_M_r T)
    rc=$?
    debug "turn off automation: rc=$rc, $result"

    if [ $rc -ne 0 ]; then
        printerrmsg 3
        log_and_exit "Cannot disable HA environment."
    fi

    set_crit_rsrc_protection $RSCT_CRIT_RSRC_PROTECT__OFF

    # set flag so post and cleanup phases know what to do

    touch $RESTART_AUTOMATION_FLAGFILE
}

# the following function stops the RSCT stack, which may include TSA, and checks for errors

stop_rsct () {
    local rc
    local result

    result=$(rmcctrl -Z 2>&1)
    rc=$?
    debug "stop RSCT stack: rc=$rc, $result"

    if [ $rc -ne 0 ]; then
        printerrmsg 3
        log_and_exit "Cannot stop RSCT."
    fi

    # set flag so post and cleanup phases know what to do

    touch $RESTART_RSCT_FLAGFILE
}

# The following function restarts RSCT and re-enables automation

restart_RSCT_Automation () {
    local rc
    local result

    if [ -f $RESTART_RSCT_FLAGFILE ]; then

        result=$(rmcctrl -s 2>&1)
        rc=$?
        debug "start RSCT stack: rc=$rc, $result"

        rm -f $RESTART_RSCT_FLAGFILE
    fi

    if [ -f $RESTART_AUTOMATION_FLAGFILE ]; then

        wait_for_domain
        rc=$?

        if [ $rc -eq 0 ]; then

            # wait for the HA status to be Active once more, before turning automation back on
            # However, we don't wait forever. If this node does not go back to Active, re-enable
            # automation and let TSA do its thing, which could be a failover
            #
            # Also exit the loop if the node becomes Passive. This could occur if this node
            # crashed between the pre and post steps, then the other node was "forced Active"

            for i in {1..6}
            do
                smhastatus >& /dev/null
                rc=$?
                if [ $rc -eq 0 -o $rc -eq 3 ]; then
                    break;
                fi
                sleep 10
            done

            set_crit_rsrc_protection $RSCT_CRIT_RSRC_PROTECT__HARD_RESET

            result=$(call_tsa_samctrl_M_r F)
            rc=$?
            debug "turn on automation: rc=$rc, $result"

            # include_peer_node
            # delete_exclude_peer_flagfile
        fi

        rm -f $RESTART_AUTOMATION_FLAGFILE
    fi
}


RC=0              # assume everything is ok 

CHKTIME_CMD=$1
CHKTIME_PHASE=$2

debug "Called with $*"

shift 2

if [ "$CHKTIME_CMD" != "chconfig.d" -o -z "$CHKTIME_PHASE" ]
then
    printerrmsg 4
    log_and_exit "Plug-in called with invalid arguments."
fi

case $CHKTIME_PHASE in
    pre)

        if rw_lock write get CmdSMHA $PROGNAME 10
        then
            SMHACHKTIME_ACQUIRED_LOCK=yes

            if files_exist $HACMD_INPROGRESS ; then
                haRC=9    # return "busy" error if another command is in progress
            else

                # check HA status

                smhastatus >& /dev/null
                haRC=$?
            fi
        else
            haRC=9    # return "busy" error if cannot get lock
        fi

        if chk_for_negative_time $* ; then
            case $haRC in
                0 )              # Active node
                   stop_automation
                   stop_rsct
                   touch ${HACMD_INPROGRESS}_chktime           # lock out calls to certain HA commands
                   ;;

                3 | 4 | 7 )              # Passive node or HA is not configured or HA is stopped
                   stop_rsct
                   touch ${HACMD_INPROGRESS}_chktime           # lock out calls to certain HA commands
                   ;;

                * )
                   printerrmsg 5
                   log_and_exit "Cannot change system time backward; HA is busy."
                   ;;
            esac
        else
            case $haRC in
                0 | 3 | 4 | 7 )          # Active or Passive node or HA is not configured or HA is stopped
                   debug "Time moving forward: is active/passive/not_configured/stopped"
                   ;;

                * )
                   printerrmsg 5
                   log_and_exit "Cannot change system time forward; HA is busy."
                   ;;
            esac
        fi
        ;;
    post | cleanup)
        if rw_lock write get CmdSMHA $PROGNAME 100000     # wait "forever"
        then
            SMHACHKTIME_ACQUIRED_LOCK=yes

            rm -f ${HACMD_INPROGRESS}_chktime        # allow calls to smha commands once write lock is freed
        fi

        restart_RSCT_Automation
        ;;
    *)
        printerrmsg 4
        log_and_exit "Plug-in called with invalid arguments."
        ;;
esac

if [ "$SMHACHKTIME_ACQUIRED_LOCK" = "yes" ]; then
    rw_lock write free CmdSMHA $PROGNAME
fi

debug "Exiting script with $RC"

exit $RC
