#!/bin/bash
# Licensed Materials - Property of IBM
# 
# (C) Copyright IBM Corp. 2009, 2013 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
# IBM_COPYRIGHT_END 


PATH=/usr/bin:$PATH
PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)

INSTALL_DIR=/opt/ibm/director
CMD_DIR=$INSTALL_DIR/ha/bin/priv
COMMON_CODE=/opt/ibm/director/ha/bin/include/smhacommon.sh

if [ -f $COMMON_CODE ]; then
    source $COMMON_CODE
else
    echo "$PROGNAME:Common code not found."
    exit 1
fi
#############################################################
load_common_code $INSTALL_DIR/ha/bin/include/smhaerrcode.sh

PARM_FLAGS="h help u users f firewall r prep d drbd t terminate T tsa a authentication-mech c update-interfaces N ntp E ntp-key-file B dns y syslog z discard-floating-addrs G groups m mirror-required-passive-dirs"
PARM_MULTI="network-alias network-alias6 network-check-addr"
PARM_TRANSLATION="n node \
N ntp \
E ntp-key-file \
B dns \ 
p password \
u users \
U user \
C cuser \
f firewall \
D dhcp \
d drbd \
L local-replication-address \
R remote-replication-address \
i network-alias \
I network-alias6 \
q network-quorum \
g agent-manager-address \
G groups \
r prep \
x node-id \
t terminate \
T tsa \
a authentication-mech \
A activations \
c update-interfaces \
y syslog \
h help \
z discard-floating-addrs \
o old-hostname \
m mirror-required-passive-dirs \
P primary-node-cas-ip \
S secondary-node-cas-ip \
H network-check-addr"
PARM_SINGLES="help users firewall authentication-mech update-interfaces dns ntp ntp-key-file syslog groups mirror-required-passive-dirs"
PARM_VALID=$PARM_TRANSLATION
PARM_DONT_SAVE="help tsa password cuser discard-floating-addrs"

load_common_code $INSTALL_DIR/ha/bin/include/smhaparms.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhadrbd.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhadir.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhadb2.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhamodel.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhacim.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhaplugin.sh
load_common_code $INSTALL_DIR/ha/bin/include/smharsrc.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhaarp.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhadhcp.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhaagent.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhaerrcode.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhainitcom.sh


###################################
## Functions                     ##
###################################

# the following function is used to update the configuration status file with
# error status whenever this script exits with an error. The trap exit handler and
# the exit trap status message is set at the beginning of each processing function.
# The exit trap is reset before the process terminates with a zero exit status.

EXIT_TRAP_MESSAGE=
EXIT_TRAP_MSGKEY=
EXIT_TRAP_STEP=

trap_on_exit () {
    debug "Trap on exit at configuration step $EXIT_TRAP_STEP"
    sm_msg "${EXIT_TRAP_MESSAGE}"

    # record error status and send to other node 
    record_cfg_status_fatal_error "${EXIT_TRAP_MSGKEY}"
}

set_exit_trap () {
    local cfgStep=$1

    if [ $cfgStep -eq 12 ]; then
        EXIT_TRAP_MESSAGE="`eval_gettext "The high availability configuration failed.  Restore both nodes from backup and try again."`"
        EXIT_TRAP_MSGKEY="The high availability configuration failed.  Restore both nodes from backup and try again."
    elif [ -z "$EXIT_TRAP_MSGKEY" ]; then
        EXIT_TRAP_MESSAGE="`eval_gettext "The high availability configuration failed.  Try the configuration again.  If it still fails, restore both nodes from backup and try again."`"
        EXIT_TRAP_MSGKEY="The high availability configuration failed.  Try the configuration again.  If it still fails, restore both nodes from backup and try again."
    fi

    EXIT_TRAP_STEP=$cfgStep

    debug "Set config exit trap for step ${EXIT_TRAP_STEP}"

    trap trap_on_exit EXIT
}

reset_exit_trap () {
    local RC=$1

    if [ $RC -eq 0 ]; then
        trap - EXIT           # remove exit trap if exit code is 0
    fi
}

proc_parms () {
    local rc=0
    
    # Parse the parameters
    # The result should be that variables are set for each parameter and
    # CMD_PARMS will contain the list of parameters processed
    # If a file is specified it will also read the file.
    parse_all_parms $*    
        
    if [ -n "$local_replication_address" ]; then
        if get_canonical_IP $local_replication_address ; then
            debug "Normalize ${local_replication_address} to ${NORMALIZED_IP_ADDRESS}"
            local_replication_address=$NORMALIZED_IP_ADDRESS
        else
            rc=1
        fi
    fi

    if [ -n "$remote_replication_address" ]; then
        if get_canonical_IP $remote_replication_address ; then
            debug "Normalize ${remote_replication_address} to ${NORMALIZED_IP_ADDRESS}"
            remote_replication_address=$NORMALIZED_IP_ADDRESS
        else
            rc=2
        fi
    fi

    if [ $rc -ne 0 ]; then
        set_exit_trap 99
        if [ $rc -eq 1 ]; then
            fatal_error "Could not normalize local IP address $local_replication_address"
        else
            fatal_error "Could not normalize remote IP address $remote_replication_address"
        fi
    fi
}

# requires that $HOST and $node are already set.
create_keystores() {
	
	    local LOCAL_NODE=$HOST
        local REMOTE_NODE=$node
        local KEYSTORE_PASS=ibmpassw0rd
        local SIG_ALG=SHA256withRSA
        local KEYSIZE=2048
       	
       	local LOCAL_BROKER_FILE=$BROKER_KS
       	local LOCAL_BROKER_CERT=$SMHA_CONFIGDIR/local_broker_cert
       	local REMOTE_BROKER_FILE=$SMHA_CONFIGDIR/remote_broker.ks
       	local REMOTE_BROKER_CERT=$SMHA_CONFIGDIR/remote_broker_cert
       	
       	if [ -f $LOCAL_BROKER_FILE ]; then
        	rm $LOCAL_BROKER_FILE
    	fi
    	if [ -f $LOCAL_BROKER_CERT ]; then
        	rm $LOCAL_BROKER_CERT
    	fi
    	if [ -f $REMOTE_BROKER_FILE ]; then
        	rm $REMOTE_BROKER_FILE
    	fi
    	if [ -f $REMOTE_BROKER_CERT ]; then
        	rm $REMOTE_BROKER_CERT
    	fi
    	
       	# Change the mask so that new files aren't world readable.
       	local mask=`umask`
       	umask 077
       	
       	# create local keystore and export the CA
		$JREBIN_DIR/keytool -genkeypair -alias broker -keyalg RSA -validity 9999 -dname CN=$LOCAL_NODE,O=IBM -keystore $LOCAL_BROKER_FILE -storepass $KEYSTORE_PASS -keypass $KEYSTORE_PASS -sigalg $SIG_ALG -keysize $KEYSIZE
		 RC=$?
        if [ $RC -ne 0 ]; then
            return_error $SSL_KEYS_NOT_CREATED "Error creating local host self signed cert RC=$RC"
        fi
                
		$JREBIN_DIR/keytool -export -alias broker -keystore $LOCAL_BROKER_FILE -storepass $KEYSTORE_PASS -file $LOCAL_BROKER_CERT
		 RC=$?
        if [ $RC -ne 0 ]; then
            return_error $SSL_KEYS_NOT_CREATED "Error exporting local host cert RC=$RC"
        fi
        
		# create remote keystore and export it's CA
		$JREBIN_DIR/keytool -genkeypair -alias broker -keyalg RSA  -validity 9999  -dname CN=$REMOTE_NODE,O=IBM -keystore $REMOTE_BROKER_FILE -storepass $KEYSTORE_PASS -keypass $KEYSTORE_PASS -sigalg $SIG_ALG -keysize $KEYSIZE
		 RC=$?
        if [ $RC -ne 0 ]; then
            return_error $SSL_KEYS_NOT_CREATED "Error creating remote host self signed cert RC=$RC"
        fi
        
		$JREBIN_DIR/keytool -export -alias broker -keystore $REMOTE_BROKER_FILE -storepass $KEYSTORE_PASS -file $REMOTE_BROKER_CERT
	 	RC=$?
        if [ $RC -ne 0 ]; then
            return_error $SSL_KEYS_NOT_CREATED "Error exporting remote host cert RC=$RC"
        fi
        
		# import the CA from the remote into the local Keystore
		$JREBIN_DIR/keytool -import -noprompt -alias client -keystore $LOCAL_BROKER_FILE -storepass $KEYSTORE_PASS -trustcacerts -file $REMOTE_BROKER_CERT
		 RC=$?
        if [ $RC -ne 0 ]; then
            return_error $SSL_KEYS_NOT_CREATED "Error importing remote cert into local keystore RC=$RC"
        fi
        
		# import the CA from the local into the remote Keystore
		$JREBIN_DIR/keytool -import -noprompt -alias client -keystore $REMOTE_BROKER_FILE -storepass $KEYSTORE_PASS -trustcacerts -file $LOCAL_BROKER_CERT
       	 RC=$?
        if [ $RC -ne 0 ]; then
            return_error $SSL_KEYS_NOT_CREATED "Error importing local cert into remote keystore RC=$RC"
        fi
       	
       	# delete the cert files, we don't need them anymore
		rm $LOCAL_BROKER_CERT
		rm $REMOTE_BROKER_CERT
 
        umask $mask
              	
       	return $RC
       	
}


#####################################################
# Process the help parameter
#####################################################
proc_help () {
    
    if [[ "$help" == "Y" ]]; then
        echo_help
        exit 0
    fi

}

echo_help () {
        echo "${PROGNAME}: [-h][-n <node>]"
        echo ""
        echo "The ${PROGNAME} command is used to start the HA daemon."
        echo "Options:"
        echo ""
        echo "  -h,--help                       Display this help"
        echo "  -n,--node                       Specifies the companion node in the configuration"
        echo " "
}

#####################################################
# Process the node parameter
#####################################################
proc_node () {
    
    # if the node parameter was specified then handle it.
    if [ -n "$node" ]; then
    	
    	# Remove any MQ data
        rm -rf $SMHA_DIR/persist/*
        # Remove files indicating previous HA config error (if they exist)
        rm -rf $ERROR_DURING_CONFIG_FLAGFILE
        rm -rf $CFG_STATUS_FILE
        #rm -rf $DM_CONF # Don't remove DM_CONF file here, because it causes the HA Daemon to go into an error state on Replace Passive Node
        rm -rf $BROKER_KS
        
    	# create keystores for both nodes
        create_keystores
        
        RC=$?
        if [ $RC -ne 0 ]; then
            return_error $SSL_KEYS_NOT_CREATED "Error setting up keystores RC=$RC"
        fi
    	
    	# start the mq daemon now - takes longer
        /etc/rc.d/smhamq restart
       	
       	wait_for_mq_online
       	RC=$?
       	if [ $RC -ne 0 ]; then
            return_error $MQ_NOT_STARTING "MQ not coming online. RC=$RC"
        fi
        set_exit_trap 1
       	       	
    	# Run the code to initialize HA
    	if [ -z "$user" ]; then
    		get_admin_id
    		user=$ADMIN_ID
    	fi
    	
    	if [ -z "$cuser" ]; then
    		cuser=$user
    	fi
    	
    	save_cfg_user_id $cuser
    	run_init "init" $node $user $password $cuser $secondary_node_cas_ip $activations
    	RC=$?

        # if primary_node_cas_ip was specified, set it now, else assume this
        # call is a part of replace a node and this property has already been set
        if [ -n "${primary_node_cas_ip}" ]; then
            set_cas_ip_property ${primary_node_cas_ip} restartAgent
        fi

        # check for network-check-addr parm(s). For now, it is optional.

        if [ -n "${network_check_addr__1}" ]; then
            rm -f $RSCT_NETMON_CF_FILE

            local nca
            local nca_value
            local i

            for i in {1..20}; do
                nca=network_check_addr__${i}
                nca_value=${!nca}
                if [ -n "$nca_value" ]; then
                    echo "$nca_value" >> $RSCT_NETMON_CF_FILE
                else
                    # no more parms found
                    break
                fi
            done

            # send the file to the secondary node
    	    run_init "netmon"
    	    RC=$?
            if [ $RC -ne 0 ]; then
                debug "run_init failed with RC=$RC"
                exit $RC
            fi
        else
            debug "Missing at least one --network-check-addr; in the future, call return_error here."
        fi

        # See if rmcd is bound to TCP and UDP ports 657
        debug "Checking what process is bound to 657 TCP and/or UDP"
        portInUse=`netstat -tulpn | grep 657`
        debug "${portInUse}"
        
        # Force rmcd to bind to TCP and UDP ports 657 if it isn't already
        # For some reason rmcd isn't always bound to these ports, so we
        # added this to explicitly try it so HA config won't fail later
        # when we run preprpnode and mkrpdomain
        /usr/bin/rmcctrl -p
        
        # TRANSLATORS: The "resources" specified in the message refer to hardware and
        # software components such as disks, memory, software programs such as databases,
        # ftp servers, CIM servers, etc.  These resources are being synchronized so they
        # are the same or similar on both nodes.
        sm_msg "`eval_gettext "Synchronizing resources."`"
        record_cfg_status "Synchronizing resources."

        get_OEM_dirc_images
    	
        reset_exit_trap $RC
        exit $RC
    fi
    
}


#####################################################
# Process the authentication-mech parameter
#####################################################
proc_authentication_mech () {
    
    # if the authentication-mech parameter was specified then handle it.
    if [ -n "$authentication_mech" ]; then
        set_exit_trap 6
        
        # Run the code to sync remote authentication mechanism configuration files (kerberos and LDAP)
        run_init "authentication_mech" 
        RC=$?
        
        reset_exit_trap $RC
        exit $RC
    fi
    
}



#####################################################
# Process the dns parameter
#####################################################
proc_dns () {
    
    # if the dns parameter was specified then handle it.
    if [ -n "$dns" ]; then
        set_exit_trap 3
        
        # Run the code to sync the DNS BIND resolver file (/etc/resolv.conf) 
        run_init "dns" 
        RC=$?
        
        reset_exit_trap $RC
        exit $RC
    fi
    
}

#####################################################
# Process the mirror-required-passive-dirs parameter
#####################################################
proc_mirror_required_passive_dirs () {
    
    # if the mirror-required-passive-dirs parameter was specified then handle it.
    if [ -n "$mirror_required_passive_dirs" ]; then
        set_exit_trap 9
        
    	# Generate three filenames that are  unique enough to use
	    # smhaoverrides-procid-date-time
        # smhalwisecurity-procid-date-time
        # smhadatasecurity-procid-date-time
        # TRANSLATORS: Ignore this, it should not be translated
	    zipname1="/tmp/smhaoverrides"$$"-"
        zipname2="/tmp/smhalwisecurity"$$"-"
        zipname3="/tmp/smhadatasecurity"$$"-"
        zipname4="/tmp/smhaamcerts"$$"-"
	    dt=`date -u '+%F-%T'`
	    zipname1=${zipname1}${dt}".zip"
        zipname2=${zipname2}${dt}".zip"
        zipname3=${zipname3}${dt}".zip"
        zipname4=${zipname4}${dt}".zip"
        rc1=0
        rc2=0
        rc3=0
        rc4=0

	    # I don't want fully qualified path names in the zip file.  I want things relative to the directory.
        currentwd=`pwd`
        target1=/opt/ibm/director/lwi/conf/overrides
        if [[ -e $target1 ]]; then
	    cd $target1
            debug "creating zip file ${zipname1}"
	    zip -ry ${zipname1} . 1>/dev/null
            rc1=$?
            debug "zip of overrides directory returned: $rc1"
        else
            debug "$target1 does not exist!"
            rc1=1
        fi

        target2=/opt/ibm/director/lwi/security
        if [[ -e $target2 ]]; then
            cd $target2
            debug "creating zip file ${zipname2}"
            zip -ry ${zipname2} . 1>/dev/null
            rc2=$?
            debug "zip of lwi/security directory returned: $rc2"
        else
            debug "$target2 does not exist!"
            rc2=2
        fi

        target3=/opt/ibm/director/data/security
        if [[ -e $target3 ]]; then
            cd $target3
            debug "creating zip file ${zipname3}"
            zip -ry ${zipname3} . 1>/dev/null
            rc3=$?
            debug "zip of data/security directory returned: $rc3"
        else
            debug "$target3 does not exist!"
            rc3=3
        fi
        target4=/opt/ibm/director/lwi/runtime/agentmanager/eclipse/plugins/com.ibm.tivoli.cas.manager/certs
        if [[ -e $target4 ]]; then
            cd $target4
            debug "creating zip file ${zipname4}"
            zip -ry ${zipname4} . 1>/dev/null
            rc4=$?
            debug "zip of com.ibm.tivoli.cas.manager/certs directory returned: $rc4"
        else
            debug "$target4 does not exist!"
            rc4=4
        fi
	    cd ${currentwd}
		
        # Run the code to mirror the directories to the passive node
        if [[ $rc1 -eq 0 && $rc2 -eq 0 && $rc3 -eq 0 && $rc4 -eq 0 ]]; then
            run_init "mirror-required-passive-dirs" ${zipname1} ${target1} ${zipname2} ${target2} ${zipname3} ${target3} ${zipname4} ${target4}
            RC=$?
        else
            debug "proc_mirror_required_passive_dirs: failed to zip passive directories: $rc1 $rc2 $rc3 $rc4"
            RC=$UNKNOWN_ERROR
        fi

        rm -rf ${zipname1}
        rm -rf ${zipname2}
        rm -rf ${zipname3}
        rm -rf ${zipname4}

        debug "proc_mirror_required_passive_dirs: returning with exit code $RC"
        reset_exit_trap $RC
        exit $RC
    fi
    
}
#####################################################
# Process the syslog parameter
#####################################################
proc_syslog () {
    
    # if the syslog parameter was specified then handle it.
    if [ -n "$syslog" ]; then
        set_exit_trap 5
        
    	# Generate a filename that is unique enough to use
		# smhasyslog-procid-date-time
		# TRANSLATORS: Ignore this, it should not be translated
		zipname="/tmp/smhasyslog"$$"-"
		dt=`date -u '+%F-%T'`
		zipname=${zipname}${dt}".zip"
		debug "creating zip file ${zipname}"
		# I don't want fully qualified path names in the zip file.  I want things relative to the directory.
		currentwd=`pwd`
		cd /etc/rsyslog.d/
		zip -ry ${zipname} .
		cd ${currentwd}
		debug "zip returned: $?"
        # Run the code to sync the sylog config file (/etc/syslog.conf) 
        run_init "syslog" ${zipname}
        RC=$?
        rm -rf ${zipname}
        reset_exit_trap $RC
        exit $RC
    fi
    
}

#####################################################
# Process the ntp parameter
#####################################################
proc_ntp () {
    
    # if the ntp parameter was specified then handle it.
    if [ -n "$ntp" ]; then
        set_exit_trap 4
        
        # Run the code to sync the NTP files (/etc/ntp.conf and /etc/ntp.keys) 
        run_init "ntp" 
        RC=$?
        
        reset_exit_trap $RC
        exit $RC
    fi
    
}

#####################################################
# Process the ntp parameter
#####################################################
proc_ntp_key_file () {
    
    # if the ntp parameter was specified then handle it.
    if [ -n "$ntp_key_file" ]; then
        set_exit_trap 4
        
        # Run the code to sync the NTP key file (/etc/ntp.keys) 
        run_init "ntp-key-file" 
        RC=$?
        
        reset_exit_trap $RC
        exit $RC
    fi
    
}

#####################################################
# Process the groups parameter
#####################################################
proc_groups () {
    
    # if the groups parameter was specified then handle it.
    if [ -n "$groups" ]; then
        set_exit_trap 7
        
        # Run the code to sync users
        run_init "groups" 
        RC=$?
        
        reset_exit_trap $RC
        exit $RC
    fi
    
}

#####################################################
# Process the users parameter
#####################################################
proc_users () {
    
    # if the users parameter was specified then handle it.
    if [ -n "$users" ]; then
        set_exit_trap 8
        
        # Run the code to sync users
        run_init "users" 
        RC=$?
        
        reset_exit_trap $RC
        exit $RC
    fi
    
}

#####################################################
# Process the firewall parameter
#####################################################
proc_firewall () {
    
    # if the firewall parameter was specified then handle it.
    if [ -n "$firewall" ]; then
        set_exit_trap 2
        
        # Run the code to sync the firewall
        run_init "firewall" 
        RC=$?
        
        reset_exit_trap $RC
        exit $RC
    fi
    
}

#####################################################
# Process the update-interfaces parameter
#####################################################
proc_update_interfaces () {
    
    # if the firewall parameter was specified then handle it.
    if [ -n "$update_interfaces" ]; then
        
        # Run the code to update just the firewall interfaces
        run_init "update-interfaces" 
        RC=$?
        
        exit $RC
    fi
    
}

#####################################################
# Process the prep parameter
#####################################################
proc_prep () {
    
    # if the prep parameter was specified then handle it.
    if [ -n "$prep" ]; then
        set_exit_trap 11
        
        # TRANSLATORS: The "nodes" specified in the message refer to the two
        # systems that will be joined to the high availability environment.
        # "cluster" is a high availability term that refers the group of
        # systems that form the high availability environment
        sm_msg "`eval_gettext "Preparing the nodes to join the cluster."`"
        record_cfg_status "Preparing the nodes to join the cluster."
        # Run the code to initialize HA
        run_init "prep" $node_id
        RC=$?
        
        reset_exit_trap $RC
        exit $RC
    fi
    
}

#####################################################
# Process the drbd parameter
#####################################################
proc_drbd () {
    
    # if the drbd parameter was specified then handle it.
    if [ -n "$drbd" ]; then
        set_exit_trap 10
        
        # Run the code to initialize HA
        run_init "drbd"  $local_replication_address $remote_replication_address
        RC=$?
        
        reset_exit_trap $RC
        exit $RC
    fi
    
}

#####################################################
# Process the terminate parameter
#####################################################
proc_term () {
    
    # if the terminate parameter was specified then handle it.
    if [ -n "$terminate" ]; then

        debug "Starting termination code."
            
        # Run the code to terminate HA
        record_starting_termination
        
        run_conf_plugins unconf begin
        
        # Only run the init if our mq is running, otherwise we'll
        # hang trying to send a message and HA is probably not
        # configured anyway.
        /etc/rc.d/smhamq status
        if [ $? -eq 0 ]; then
            if [ -n $activations ]; then
              run_init "term" -A $activations
            else
              run_init "term"
            fi
        fi
        
        delete_tsa_model
        
        discardFloatingAddrsParm=
        if [ -n "$discard_floating_addrs" ]; then
            discardFloatingAddrsParm="--discard-floating-addrs"
        fi
        
        # Do any final termination code.  This is common between nodes
        if [ -n "$activations" ]; then
            $SMHA_SCRIPTS/priv/smhaterm.sh --restore -A $activations $discardFloatingAddrsParm
        else
            $SMHA_SCRIPTS/priv/smhaterm.sh --restore $discardFloatingAddrsParm
        fi
        
        RC=$?
        
        exit $RC
    fi
    
}

#####################################################
# Process the dhcp parameter
#####################################################
proc_dhcp_on_off () {
    
    # if the dhcp parameter was specified without the full TSA setup, then handle it.
    if [ -n "$dhcp" ]; then
        RC=0
        check_key_values dhcp enable disable
        
        debug "Processing dhcp command:$dhcp"
        is_domain_setup
        if [ $? -eq 1 ]; then
        	# Don't allow dhcp to be enabled/disabled unless it was previously setup
        	get_value_from_cfg dhcp
        	local share_val=$CFG_RETURN_VALUE        
            if [ -n "$share_val" ]; then

	            case $dhcp in
	            "enable" )
                    get_value_from_cfg node
    	            node=$CFG_RETURN_VALUE        
	            
	                # create the resources
	                crt_dhcp_rsrc $share_val
	                crt_dhcp_rsrc_step2
	                sleep 5
	                # chrg -o online $DHCP_RG
                        debug "tsa call hook 12"
                        local tsa_return
	                tsa_return=$(call_tsa_chrg_o_l_u -o online $DHCP_RG)
	                RC=$?
	                if [ $RC -ne 0 ]; then
	                    return_error $RC "Error making $DHCP_RG online. RC=$RC $tsa_return"
	                fi
	                
	            ;;
	            "disable" )
	                dlt_dhcp_rsrc
	            ;;
	            esac
            	        	        	
            else
                return_error $DHCP_WAS_NOT_CONFIGURED_WITH_HA "DHCP Services were not configured with HA."            
            fi
        	        	
        else
            return_error $HA_ENVIRONMENT_NOT_CONFIGURED "The HA environment is not configured."
        fi
                
        exit $RC
    fi
    
}


#####################################################
# Process the tsa parameter
#####################################################
proc_tsa () {
    
    # if the tsa parameter was specified then handle it.
    if [ -n "$tsa" ]; then
        set_exit_trap 12
        
        # TRANSLATORS: The "local agent" specified in the message is a process
        # running on the local system that handles some systems management function
        sm_msg "`eval_gettext "Configuring local agent."`"
        record_cfg_status "Configuring local agent."
        register_meps
        turn_off_address_change_processing
        force_address_change_processing_next_restart
        
        remove_director_autostart
        save_dirstartservice_status
        remove_dirstartservice
        
        # Take down Director (Should already be done by the GUI scripts)
        debug "Stopping director."
        # (OEMB)
        # TRANSLATORS: The "management server" specified in the message is the
        # IBM Director Server, which is a process used to manage the systems
        # in your data center.  We have to use the general term "management server"
        # because different products are using this code, and some don't want
        # the term "IBM Director" to show up.
        sm_msg "`eval_gettext "Stopping the management server on the primary node."`"
        record_cfg_status "Stopping the management server on the primary node."
        $DIRECTOR_MONITOR_SCRIPT stop 
        
        # Comment out the DB2 FMC and turn off auto start
        comment_fmc
        db2auto_off
        add_ha_to_inittab
        
        # Create the flag file so when init q is run in the next step
        # the smhamaint.sh script doesn't do anything.  It needs to be
        # run now though, so it doesn't run later when some other 
        # script calls init q
        create_stop_maint_flagfile
        
        #Settings in /etc/inittab were changed. Poke init to read the new config
		init q
        
        # Stop db2 (Should already be done by the GUI scripts)
        debug "Stopping db2."
        sm_msg "`eval_gettext "Stopping the database on the primary node."`"
        record_cfg_status "Stopping the database on the primary node."
        stop_db2

        # Kill db2fmd so it unlocks files on the mirror drive
        debug "Kill db2fmd."
        kill_db2fmd
        
        deactivate_dir_server_nonstop
        
        # Create the AM GUI file from Tivoli GUID file
        /opt/tivoli/guid/tivguid -Show | grep Guid |  cut -d':' -f2 | sed -e 's/\.//g'| sed -e 's/[a-z]/\u&/g' | tr -d \\n > /etc/ibm/tivoli/AM.guid
                
        crt_tsa_model       
        
        debug "Set flag file so smhastatus doesn't return error during this time where the model is created but the resources haven't been brought online yet"
        touch $SMHA_CONFIGDIR/.modelJustCreated
        
        debug "Append to config file"
        append_string_to_cfg_file "--node-order=$HOST,$node"
        append_to_cfg_file $CMD_PARMS
        
        sm_msg "`eval_gettext "Synchronizing configuration with the secondary node."`"
        record_cfg_status "Synchronizing configuration with the secondary node."
        send_conf_parms
        RC=$?
                        
        # Add the secondary node to drbd primary so we can fail over from this point on.
        add_drbd_secondary_node
    
        # Bring Director Online
        # TRANSLATORS: The "resources" specified in the message refer to hardware and
        # software components such as disks, memory, software programs such as databases,
        # ftp servers, CIM servers, etc.  These resources are being synchronized so they
        # are the same or similar on both nodes.  These resources were just created in the
        # high availability model.
        sm_msg "`eval_gettext "Done creating the resources."`"
    
        debug "Bring Director Online"
        start_all_rg_online        


        run_conf_plugins conf end
        
        debug "--- Configuration is done ---"
        rm_cfg_status_file
        rm_cfg_user_file
        
        debug "Remove flag file indicating model is created but the resources haven't been brought online yet, since they were just brought online"
        rm -f $SMHA_CONFIGDIR/.modelJustCreated  
        # send status update to secondary node to clear the .modelJustCreated flag file      
        run_init "status" "-F"
        
        reset_exit_trap $RC
        exit $RC
    fi
    
}


###################################
## Main code                     ##
###################################

if [ $# -eq 0 ]; then
    echo_help
    exit 0
fi

proc_parms $*

debug "smhainit.sh script entered:  my process id is $$"

##
## There are only a few ways this command can be called.
## Run down through each and process them.
##

# Take care of help first
proc_help
# If tsa is specified then create the tsa model
proc_tsa
###
if [ -z "$tsa" ]; then
    # If node is specified then they are doing initial setup
    proc_node
fi
###
if [ -z "$tsa" ]; then
    # DHCP
    proc_dhcp_on_off
fi
# If dns is specified then process DNS
proc_dns
# if syslog is specified then process syslog
proc_syslog
# If ntp is specified then process NTP
proc_ntp
# If ntp_key_file is specified then process NTP
proc_ntp_key_file
# If authentication_mech is specified then sync authentication mechanisms
proc_authentication_mech 
# If groups is specified then sync the group file
proc_groups
# If users is specified then sync users
proc_users
# If firewall is specified then sync the firewall
proc_firewall
# If update-interfaces is specified the update the firewall interfaces
proc_update_interfaces
# If prep is specified then prep the nodes
proc_prep
# If drbd is specified then create the conf file
proc_drbd
# If mirror-required-passive-dirs is specified then mirror the directories to the passive node
proc_mirror_required_passive_dirs
# If terminate is specified then terminate the HA environment
proc_term

exit 0

