#!/bin/bash
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2013 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
# IBM_COPYRIGHT_END

PATH=/usr/bin:$PATH
PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)

INSTALL_DIR=/opt/ibm/director
CMD_DIR=$INSTALL_DIR/ha/bin/priv
COMMON_CODE=/opt/ibm/director/ha/bin/include/smhacommon.sh

if [ -f $COMMON_CODE ]; then
    source $COMMON_CODE
else
    echo "$PROGNAME:Common code not found."
    exit 1
fi
#############################################################

PING6=/bin/ping6
PING4=/bin/ping

# This script is expected to return in a short time. It has been observed
# that the ping command can hang. Therefore, this script makes a "recursive"
# call to itself, where the second instance (child process) serves as a
# watchdog timer. If the timer pops, SIGSYS is sent to the parent process.
#
# N.B. SIGSYS is used, since IBM.ConfigRM blocks all asynchronous signals
# and bash does not permit an inherited blocked signal to be trapped.

alarm_trap () {
    # get some debug info for the ping command

    cmd_out=$(echo ""; LC_ALL=C ps -lp $CHILDPID 2>&1)
    debug "caught SIGSYS, ps -lp $CHILDPID: $cmd_out"

    kill -9 $CHILDPID >& /dev/null
}

# set the trap

trap alarm_trap SIGSYS

# Get command arguments

Operation=$1
Address=$2
token=${3#\"}
IPAddr=${token%\"}

debug "Operation=$Operation Address=$Address IPAddr=$IPAddr"

if [ "$Operation" != "reserve" -a "$Operation" != "initialize" -a "$Operation" != "WatchDog" ]; then
    # return success for everything other than reserve and initialize and WatchDog
    exit 0
fi

if [ "$Operation" = "WatchDog" ]; then

    # close the stdout/stderr inherited from the parent process by 
    # re-assigning to /dev/null. This is necessary since it most cases the
    # parent exits long before the watchdog child exits and we don't want
    # our "grandparent" (IBM.ConfigRMd) to wait for the stdout/stderr from its
    # child to close

    exec >& /dev/null

    # this is the child process; convert args

    Ppid=$Address
    Pstart=$IPAddr

    sleep 10    # give the parent 10 seconds to do his thing

    # see if parent still exists

    Start="$(ps --no-headers -p $Ppid -o start 2> /dev/null)"

    if [ -n "$Start" -a "$Start" = "$Pstart" ]; then
        debug "send signal to parent process $Ppid"
        kill -SIGSYS $Ppid
    fi

    debug "exiting watchdog"
    exit 0
fi

# Get process ID and start time to pass to watchdog process

MyPid=$$
MyStart="$(ps --no-headers -p $MyPid -o start)"

# Start watchdog

${SMHA_SCRIPTS}/priv/smhanetq.sh WatchDog $MyPid $MyStart &

# determine if this is an IPv6 or IPv4 address

token=${IPAddr#*:}
if [ "$token" != "$IPAddr" ]; then
    PING=$PING6
else
    PING=$PING4
fi

# if [ -f /tmp/Kgnmas/TBpingcount ]; then
#    read Count junk < /tmp/Kgnmas/TBpingcount
# else
#    Count=1
# fi
Count=1

debug "Call $PING -c $Count $IPAddr"

PINGOUT=/tmp/${PROGNAME}.${MyPid}
$PING -c $Count $IPAddr >& $PINGOUT &
CHILDPID=$!

debug "waiting for $PING or trap, CHILDPID=$CHILDPID"
wait $CHILDPID
RC=$?

debug "$PING wait status is $RC, ping out: $(cat $PINGOUT 2>&1)"
rm -f $PINGOUT

exit $RC
