#!/bin/bash
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2009, 2013 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
# IBM_COPYRIGHT_END

# 
# 09N6996
#

PATH=/usr/bin:$PATH
PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)

INSTALL_DIR=/opt/ibm/director
CMD_DIR=$INSTALL_DIR/ha/bin/priv
COMMON_CODE=/opt/ibm/director/ha/bin/include/smhacommon.sh

if [ -f $COMMON_CODE ]; then
    source $COMMON_CODE
else
    echo "$PROGNAME:Common code not found."
    exit 1
fi
#############################################################

# N.B. Spelling of long form of -A is capitalized, to differentiate from "activations"
# taken from SMHA_CFG_FILE

PARM_FLAGS="h help I init-nodes X replace-node N copy-ntp-conf"
PARM_MULTI=""
PARM_TRANSLATION="n node \
f parameter-file \
p password \
X replace-node \
N copy-ntp-conf \
I init-nodes \
A Activations \
h help \
S secondary-node-cas-ip"
PARM_SINGLES="help"
PARM_VALID=$PARM_TRANSLATION
PARM_DONT_SAVE="help"


load_common_code $INSTALL_DIR/ha/bin/include/smhaparms.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhadb2.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhadir.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhaagent.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhainitcom.sh

INIT_CMD=$CMD_DIR/smhainit.sh

###################################
## Functions                     ##
###################################
proc_parms () {
    
    # Parse the parameters
    # The result should be that variables are set for each parameter and
    # CMD_PARMS will contain the list of parameters processed
    # If a file is specified it will also read the file.
    parse_all_parms $*
    
    # this is only for debug
    # list_parm_vars
      
    for option in $CMD_PARMS
    do
#       echo "option: $option"
        # For each option, make sure the minimum parameters are specified
## !!!! Checking is not complete....just enough to check the code is done       
        case $option in 
            replace_node)
               ensure_parms "password"
            ;;
            init_nodes)
               ensure_parms "password"
            ;;
            *)
                # nothing to do for all other parameters
                echo " " >> /dev/null
            ;;
        esac
    done

        
}


proc_init_nodes () {
    local argA=
    
    # if the node parameter was specified then handle it.
    if [ -n "$init_nodes" ]; then
        
        debug "In init nodes script to prepare for replace node ...running"
        # read all the parameters in our config file.
        # weed out comments and unneeded parameters  
        file_parms=`awk '! /[\s]*#/ {print $0}' $SMHA_CFG_FILE | grep -v -i node-order`
        
        # We have all the parms now, setup to parse them
        PARM_VALID_CHECKS=off              
        PARM_MULTI="network-alias network-alias6"        
        parse_parms $file_parms
        PARM_VALID_CHECKS=      
        
        debug "Done reading existing config file"
        
        ######################
        # setup both nodes
        ######################

        if [ -n "$activations" ]; then
            remove_option_from_cfg_file activations      # remove old value
        fi

        if [ -n "$Activations" ]; then
            activations="$Activations"
            append_to_cfg_file activations               # save new value

            argA="-A $Activations"
        fi
        $INIT_CMD -n $node -p $password -S $secondary_node_cas_ip $argA
        RC=$?
        if [ $RC -ne 0 ]; then
            return_error $RC "Error configuring nodes. RC=$RC"
        fi

        exit 0
    fi
}

proc_replace_node () {
    
    # if the node parameter was specified then handle it.
    if [ -n "$replace_node" ]; then
        
        debug "In replace node script...running"
        # read all the parameters in our config file.
        # weed out comments and unneeded parameters  
        file_parms=`awk '! /[\s]*#/ {print $0}' $SMHA_CFG_FILE | grep -v -i node-order`
        
        # We have all the parms now, setup to parse them
        PARM_VALID_CHECKS=off              
        PARM_MULTI="network-alias network-alias6"        
        parse_parms $file_parms
        PARM_VALID_CHECKS=      
        
        debug "Done reading existing config file"
        
        ######################
        # sync the firewall
        ######################
        $INIT_CMD -f
        RC=$?
        if [ $RC -ne 0 ]; then
            return_error $RC "Error synchronizing the firewall. RC=$RC"
        fi
        
        ######################
        # sync the DNS resolver
        ######################
        $INIT_CMD -B
        RC=$?
        if [ $RC -ne 0 ]; then
            return_error $RC "Error synchronizing the DNS resolver. RC=$RC"
        fi
        
        ######################
        # sync the NTP 
        ######################
        if [ -n "$copy_ntp_conf" ]; then
            $INIT_CMD --ntp
            RC=$?
        else
            $INIT_CMD --ntp-key-file
            RC=$?
        fi    
            
        if [ $RC -ne 0 ]; then
            return_error $RC "Error synchronizing the Network Time Protocol Daemon. RC=$RC"
        fi
        
        ######################
        # sync the authentication mechanisms (kerberos/ldap)
        ######################
        $INIT_CMD -a
        RC=$?
        if [ $RC -ne 0 ]; then
            return_error $RC "Error synchronizing remote authentication configuration. RC=$RC"
        fi
        
        ######################
        # sync the users
        ######################
        $INIT_CMD -u
        RC=$?
        if [ $RC -ne 0 ]; then
            return_error $RC "Error synchronizing users. RC=$RC"
        fi

        ######################
        # sync the mirrored directories that need to be visible on the passive node
        ######################
        $INIT_CMD -m
        RC=$?
        if [ $RC -ne 0 ]; then
            return_error $RC "Error synchronizing mirrored directories. RC=$RC"
        fi
        
        ######################
        # setup drbd
        ######################
        
        # Handle defaulting the local and remote addresses
        local loc_addr
        local rmt_addr
        # if specified, then use it, otherwise default it   
        if [ -n "$local_replication_address" ]; then
            loc_addr=$local_replication_address
        else
            loc_addr=`gethostip -d $HOST`
        fi
    
        # if specified, then use it, otherwise default it
        if [ -n "$remote_replication_address" ]; then
            rmt_addr=$remote_replication_address
        else
            rmt_addr=`gethostip -d $node`
        fi
        
        $INIT_CMD -d -L $loc_addr -R $rmt_addr        
        RC=$?
        if [ $RC -ne 0 ]; then
            return_error $RC "Error setting up mirroring. RC=$RC"
        fi
        
        
        get_rsct_node_id $node
        local node_id=$NODEID
                
        ######################
        # prep the nodes
        ######################
        $INIT_CMD --prep --node-id=$node_id
        RC=$?
        if [ $RC -ne 0 ]; then
            return_error $RC "Error preparing nodes. RC=$RC"
        fi

        ######################
        # Handle dhcp
        ######################
        
        get_value_from_cfg dhcp
        debug "dhcp CFG_RETURN_VALUE=$CFG_RETURN_VALUE"    
        case $CFG_RETURN_VALUE in
        "shared" )
            debug "DHCP server config is shared"
            DHCP_NONSHARED=0
        ;;
        "nonshared" )
            debug "DHCP server config is non shared"
            DHCP_NONSHARED=1
        ;;
        esac
        
        if [ "$DHCP_NONSHARED" -eq 1 ]; then
	        # Call the secondary node to setup it's dhcp config
	        run_init "dhcp nonshared"
        fi 

        ##################################
        # Send netmon.cf to the new node
        ##################################

        if [ -s $RSCT_NETMON_CF_FILE ]; then
            debug "Send $RSCT_NETMON_CF_FILE to secondary"
            run_init "netmon"
            RC=$?
            if [ $RC -ne 0 ]; then
                return_error $RC "Error copying $RSCT_NETMON_CF_FILE. RC=$RC"
            fi              
        fi

        ######################
        # Start the domain
        ######################
        startrpdomain -L $TSA_DOMAIN
        # samctrl -M F
        debug "tsa call hook 13"
        local tsa_return
        tsa_return=$(call_tsa_samctrl_M_r F)
        RC=$?
        if [ $RC -ne 0 ]; then
            return_error $RC "Error activating TSA. RC=$RC $tsa_return"
        fi              
        
        register_mep $node
        
        ######################
        # Wait for the replaced node to come online
        ######################

        wait_for_peer_state online
        RC=$?
        if [ $RC -ne 0 ]; then
            return_error $RC "Error waiting for the passive node to become online. RC=$RC"
        fi

        ######################
        # if prior to replacing the node it was shutdown gracefully with failover, then it was excluded.
        # The node is included again when the shut down node is restarted, but that doesn't happen when
        # the node is then replaced. Make sure the replaced node is not excluded
        ######################

        local excluded_node=
        excluded_node=$(get_excluded_node_name)
        RC=$?
        if [ $RC -eq 1 ]; then
            return_error $RC "Error in determining if passive node is excluded. RC=$RC"
        fi
        if [ -n "$excluded_node" ]; then
            domain_excluded_node $excluded_node include
        fi
        	                             
        ######################
        # Send over the conf file
        # This will update the agent files
        ######################
        send_conf_parms
        RC=$?
        
        # send status update to secondary node to clear the .modelJustCreated flag file      
        run_init "status" "-F"
        
        if [ $RC -ne 0 ]; then
            return_error $RC "Error replacing the passive node. RC=$RC"
        fi              
        
        ######################
        # Set up mirror0 monitoring on the replaced node.
        ######################

        setup_monitoring_for_mirror0 1    # the argument of 1 indicates to set up monitoring on the remote node only
        RC=$?
        if [ $RC -ne 0 ]; then
            return_error $RC "Error initializing the mirrored storage monitor on the passive node. RC=$RC"
        fi
        
        ######################
        # We may have just replaced a node for an upgrade scenario.
        # In this case, the local system was probably upgraded and has 
        # different installed vs active versions for RSCT and TSA.
        # Check if RSCT and/or TSA versions need to be "pushed forward"
        ######################
        run_init clustervrm
        exit 0
    fi
}


###################################
## Main code                     ##
###################################

if [ $# -eq 0 ]; then
    exit 0
fi

proc_parms $*

##
## There are only a few ways this command can be called.
## Run down through each and process them.
##

# If init-nodes is specified then we're initializing both node to prepare for the replace
proc_init_nodes
# If replace-node is specified then we're replacing the opposite node
proc_replace_node

exit 0

