#!/bin/bash
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2010, 2011 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
# IBM_COPYRIGHT_END

# 
# 09N6996 
#

PATH=/usr/bin:$PATH
PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)

INSTALL_DIR=/opt/ibm/director
CMD_DIR=$INSTALL_DIR/ha/bin/priv
COMMON_CODE=/opt/ibm/director/ha/bin/include/smhacommon.sh

if [ -f $COMMON_CODE ]; then
    source $COMMON_CODE
else
    echo "$PROGNAME:Common code not found."
    exit 1
fi

#############################################################
load_common_code $INSTALL_DIR/ha/bin/include/smhainitcom.sh

shutdown_wo_failover () {
	local RC=0
	
	is_configured
	if [ $? -eq 1 ]; then
		# HA is configured, check if this node is active
		debug "shutdown_wo_failover"
		is_passive
	    if [ $? -eq 0 ]; then
	    	# This is the active node
	    	debug "shutdown_wo_failover -active node."
            create_exclude_peer_flagfile
            # Exclude our peer, so only the floaters will be offline
	    	exclude_peer_node
	    fi
        set_crit_rsrc_protection $RSCT_CRIT_RSRC_PROTECT__OFF           
	fi
	
	return $RC
}

shutdown_w_failover () {
    local RC=0
    
    is_configured
    if [ $? -eq 1 ]; then
        # HA is configured, check if this node is active
        is_passive
        if [ $? -eq 0 ]; then
        
            get_remote_rsct_node_name
            node=$REMOTE_RSCT_NODE_NAME
            is_node_online $node
            if [ $? -eq 1 ]; then 
                # This is the active node, and we want a fail over
                debug "shutdown_w_failover -active node."
                
                create_shutdown_halt_flagfile
                # Remove this node from the domain so all the resources are shutdown
                exclude_node_from_domain                        
                set_crit_rsrc_protection $RSCT_CRIT_RSRC_PROTECT__OFF
                sleep 20
                            
                # Wait until we're passive
                local loop_count=0
                while true; do
		            is_passive
		            if [ $? -eq 1 ]; then
	                	break
		            else
		                let "loop_count=loop_count + 1"
		                debug "Waiting for node to become passive: $loop_count"
		                if [ $loop_count -ge 200 ]; then
		                	debug "Timed out waiting to become passive."
		                	break	            	
		                else
		                   sleep 5
	                    fi
                    fi
                done
            else
                debug "shutdown_w_failover -active node, but the other node is offline."
            fi
        else
            set_crit_rsrc_protection $RSCT_CRIT_RSRC_PROTECT__OFF        
        fi
    fi
    
    return $RC
	
}

handle_update () {
	local RC=0
	local parm1=$1
		
	debug "handle_update: entered with parm $parm1"
	
	is_configured
    if [ $? -eq 1 ]; then
        # HA is configured
        case $parm1 in 
        	norestart)
        		# all we need to do is restart the MQ and HA daemons
        		debug "handle_update: restarting HA MQ daemon"
        		/etc/rc.d/smhamq restart > /dev/null
        		debug "handle_update: restarting HA Daemon"
        		/etc/rc.d/smhadm restart > /dev/null
        	;;
        	restart)
  				# the update required an appliance restart.  We may have
  				# had an RSCT or TSA updated applied. 
  				debug "handle_update: calling Initializer with clustervrm" 
    			run_init clustervrm
				RC=$?
			;;
			*)
				# unknown parm received
				RC=1
				debug "handle_update: unknown parm $parm1 :  RC=$RC"
        	;;
        esac
    fi
	debug "handle_update: returning with RC=$RC"
	return $RC
}

wait_for_passive_drbd () {
	local RC=0
	debug "wait_for_passive_drbd: entered"
	is_configured
    if [ $? -eq 1 ]; then
        # HA is configured
        is_passive
		if [ $? -eq 0 ]; then
	    	# Update manager extension waits for remote node to reboot and CAS agent to be come available,
	    	# so we do not want to wait for the node to go "offline".  Start by waiting for node to go online.
			
			#wait_for_peer_state "offline"
			#RC=$?
			#if [ $RC -eq 0 ]; then
			
				# now wait for it to go online
				wait_for_peer_state "online"
				RC=$?
				if [ $RC -eq 0 ]; then
					# now wait for DRBD base resource of the peer node to go online
					wait_for_peer_drbd_base
					RC=$?
					if [ $RC -eq 0 ]; then
						debug "wait_for_passive_drbd: Passive node drbd base resource is now online"
					fi
				fi
			#fi
		else
			debug "wait_for_passive_drbd: SEVERE - should not have been called on passive node"
			RC=2
		fi
	else
		debug "wait_for_passive_drbd: SEVERE - should be called when HA is configured"
		RC=3
	fi
	debug "wait_for_passive_drbd: return: RC=$RC"
	return $RC
}

###################################
## Main code                     ##
###################################
RC=0
debug "smhaspi called with $* "

while getopts aAbeiIsfu:w flag;do
    case $flag in
        a) turn_off_automation ;;
        A) turn_on_automation ;;
        b) create_automation_flagfile ;;
        #################################
        e) exclude_peer_node ;;
        i) include_peer_node ;;
        I) create_exclude_peer_flagfile ;;
        #################################       
        s) #shutdown w/o failover
            shutdown_wo_failover
            RC=$?
        ;;
        f) #shutdown w/ failover
            shutdown_w_failover
            RC=$?
        ;;
        u)  RESTART_PARM=$OPTARG
        	handle_update $RESTART_PARM
           	RC=$?	
        ;;
        w) wait_for_passive_drbd
           RC=$?
        ;;
    esac
done
	 
debug "smhaspi exit; RC=$RC"
exit $RC
